@@ -399,6 +399,10 @@ typedef enum VMMR0OPERATION
     VMMR0_DO_GMM_CHECK_SHARED_MODULES,
     /** Call GMMR0FindDuplicatePage. */
     VMMR0_DO_GMM_FIND_DUPLICATE_PAGE,
+    /** Call GMMR0QueryStatistics(). */
+    VMMR0_DO_GMM_QUERY_STATISTICS,
+    /** Call GMMR0ResetStatistics(). */
+    VMMR0_DO_GMM_RESET_STATISTICS,
 
     /** Set a GVMM or GMM configuration value. */
     VMMR0_DO_GCFGM_SET_VALUE,
