@@ -35,6 +35,11 @@
 
 #include <iprt/cpp/exception.h>
 
+/** @defgroup grp_rt_cpp_xml    C++ XML support
+ * @ingroup grp_rt_cpp
+ * @{
+ */
+
 /* Forwards */
 typedef struct _xmlParserInput xmlParserInput;
 typedef xmlParserInput *xmlParserInputPtr;
@@ -46,29 +51,36 @@ typedef xmlError *xmlErrorPtr;
 typedef struct _xmlAttr xmlAttr;
 typedef struct _xmlNode xmlNode;
 
+/** @} */
+
 namespace xml
 {
 
+/**
+ * @addtogroup grp_rt_cpp_xml
+ * @{
+ */
+
 // Exceptions
 //////////////////////////////////////////////////////////////////////////////
 
-class RT_DECL_CLASS LogicError : public iprt::Error
+class RT_DECL_CLASS LogicError : public RTCError
 {
 public:
 
     LogicError(const char *aMsg = NULL)
-        : iprt::Error(aMsg)
+        : RTCError(aMsg)
     {}
 
     LogicError(RT_SRC_POS_DECL);
 };
 
-class RT_DECL_CLASS RuntimeError : public iprt::Error
+class RT_DECL_CLASS RuntimeError : public RTCError
 {
 public:
 
     RuntimeError(const char *aMsg = NULL)
-        : iprt::Error(aMsg)
+        : RTCError(aMsg)
     {}
 };
 
@@ -452,8 +464,8 @@ public:
 
     const AttributeNode* findAttribute(const char *pcszMatch) const;
     bool getAttributeValue(const char *pcszMatch, const char *&ppcsz) const;
-    bool getAttributeValue(const char *pcszMatch, iprt::MiniString &str) const;
-    bool getAttributeValuePath(const char *pcszMatch, iprt::MiniString &str) const;
+    bool getAttributeValue(const char *pcszMatch, RTCString &str) const;
+    bool getAttributeValuePath(const char *pcszMatch, RTCString &str) const;
     bool getAttributeValue(const char *pcszMatch, int32_t &i) const;
     bool getAttributeValue(const char *pcszMatch, uint32_t &i) const;
     bool getAttributeValue(const char *pcszMatch, int64_t &i) const;
@@ -463,17 +475,17 @@ public:
     ElementNode* createChild(const char *pcszElementName);
 
     ContentNode* addContent(const char *pcszContent);
-    ContentNode* addContent(const iprt::MiniString &strContent)
+    ContentNode* addContent(const RTCString &strContent)
     {
         return addContent(strContent.c_str());
     }
 
     AttributeNode* setAttribute(const char *pcszName, const char *pcszValue);
-    AttributeNode* setAttribute(const char *pcszName, const iprt::MiniString &strValue)
+    AttributeNode* setAttribute(const char *pcszName, const RTCString &strValue)
     {
         return setAttribute(pcszName, strValue.c_str());
     }
-    AttributeNode* setAttributePath(const char *pcszName, const iprt::MiniString &strValue);
+    AttributeNode* setAttributePath(const char *pcszName, const RTCString &strValue);
     AttributeNode* setAttribute(const char *pcszName, int32_t i);
     AttributeNode* setAttribute(const char *pcszName, uint32_t i);
     AttributeNode* setAttribute(const char *pcszName, int64_t i);
@@ -537,7 +549,7 @@ protected:
                   const char **ppcszKey);
     AttributeNode(const AttributeNode &x);      // no copying
 
-    iprt::MiniString    m_strKey;
+    RTCString    m_strKey;
 
     friend class Node;
     friend class ElementNode;
@@ -638,7 +650,7 @@ public:
     XmlMemParser();
     ~XmlMemParser();
 
-    void read(const void* pvBuf, size_t cbSize, const iprt::MiniString &strFilename, Document &doc);
+    void read(const void* pvBuf, size_t cbSize, const RTCString &strFilename, Document &doc);
 };
 
 /*
@@ -652,7 +664,7 @@ public:
     XmlFileParser();
     ~XmlFileParser();
 
-    void read(const iprt::MiniString &strFilename, Document &doc);
+    void read(const RTCString &strFilename, Document &doc);
 
 private:
     /* Obscure class data */
@@ -733,3 +745,4 @@ private:
 } // end namespace xml
 
 #endif /* !___iprt_xml_h */
+
