@@ -1,4 +1,4 @@
-/* $Rev: 36408 $ */
+/* $Rev: 74359 $ */
 /** @file
  * VBoxGuest - Linux specifics.
  *
@@ -24,12 +24,20 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_SUP_DRV
+
 #include "the-linux-kernel.h"
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 15)
+# define VBOXGUEST_WITH_INPUT_DRIVER
+#endif
+
 #include "VBoxGuestInternal.h"
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+# include <linux/input.h>
+#endif
 #include <linux/miscdevice.h>
 #include <linux/poll.h>
-#include "version-generated.h"
-#include "product-generated.h"
+#include <VBox/version.h>
 
 #include <iprt/assert.h>
 #include <iprt/asm.h>
@@ -105,6 +113,11 @@ static void                    *g_pvMMIOBase;
 static wait_queue_head_t        g_PollEventQueue;
 /** Asynchronous notification stuff.  */
 static struct fasync_struct    *g_pFAsyncQueue;
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+/** Pre-allocated mouse status VMMDev request for use in the IRQ
+ * handler. */
+static VMMDevReqMouseStatus    *g_pMouseStatusReq;
+#endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 /** Whether we've create the logger or not. */
 static volatile bool            g_fLoggerCreated;
@@ -135,6 +148,11 @@ static unsigned int             g_iModuleMajor = 0;
 # error "CONFIG_VBOXADD_MAJOR -> CONFIG_VBOXGUEST_MAJOR"
 #endif
 
+/** The input device handle */
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+static struct input_dev        *g_pInputDevice = NULL;
+#endif
+
 /** The file_operations structure. */
 static struct file_operations   g_FileOps =
 {
@@ -197,6 +215,7 @@ static const struct pci_device_id __devinitdata g_VBoxGuestPciId[] =
 };
 MODULE_DEVICE_TABLE(pci, g_VBoxGuestPciId);
 
+static PVBOXGUESTSESSION        g_pKernelSession = NULL;
 
 /**
  * Converts a VBox status code to a linux error code.
@@ -374,6 +393,115 @@ static void vboxguestLinuxTermISR(void)
 }
 
 
+enum
+{
+    /** The minumum value our device can return */
+    RANGE_MIN = 0,
+    /** The maximum value our device can return */
+    RANGE_MAX = 0xFFFF
+};
+
+
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+/** Calls the kernel IOCtl to report mouse status to the host on behalf of
+ * our kernel session. */
+static int vboxguestLinuxSetMouseStatus(uint32_t fStatus)
+{
+    return VBoxGuestCommonIOCtl(VBOXGUEST_IOCTL_SET_MOUSE_STATUS, &g_DevExt,
+                                g_pKernelSession, &fStatus, sizeof(fStatus),
+                                NULL);
+}
+
+
+/** Called when the input device is first opened.  Sets up absolute reporting.
+ */
+static int vboxguestOpenInputDevice(struct input_dev *pDev)
+{
+    NOREF(pDev);
+    if (RT_FAILURE(vboxguestLinuxSetMouseStatus
+                                   (  VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE
+                                    | VMMDEV_MOUSE_NEW_PROTOCOL)))
+        return ENODEV;
+    return 0;
+}
+
+
+/** Called if all open handles to the device are closed, disables absolute
+ * reporting. */
+static void vboxguestCloseInputDevice(struct input_dev *pDev)
+{
+    NOREF(pDev);
+    vboxguestLinuxSetMouseStatus(0);
+}
+
+
+/**
+ * Creates the kernel input device.
+ */
+static int __init vboxguestLinuxCreateInputDevice(void)
+{
+    int rc;
+
+    rc = VbglGRAlloc((VMMDevRequestHeader **)&g_pMouseStatusReq,
+                     sizeof(*g_pMouseStatusReq),
+                     VMMDevReq_GetMouseStatus);
+    if (RT_FAILURE(rc))
+        return -ENOMEM;
+    g_pInputDevice = input_allocate_device();
+    if (!g_pInputDevice)
+    {
+        VbglGRFree(&g_pMouseStatusReq->header);
+        return -ENOMEM;
+    }
+    g_pInputDevice->id.bustype = BUS_PCI;
+    g_pInputDevice->id.vendor  = VMMDEV_VENDORID;
+    g_pInputDevice->id.product = VMMDEV_DEVICEID;
+    g_pInputDevice->id.version = VBOX_SHORT_VERSION;
+    g_pInputDevice->open       = vboxguestOpenInputDevice;
+    g_pInputDevice->close      = vboxguestCloseInputDevice;
+#if 0  /* The device registration code was not backported */
+# if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+    g_pInputDevice->cdev.dev   = &g_pPciDev->dev;
+# else
+    g_pInputDevice->dev.parent = &g_pPciDev->dev;
+# endif
+#endif
+    {
+        int rc = input_register_device(g_pInputDevice);
+        if (rc)
+        {
+            VbglGRFree(&g_pMouseStatusReq->header);
+            input_free_device(g_pInputDevice);
+            return rc;
+        }
+    }
+    /* Do what one of our competitors apparently does as that works. */
+    ASMBitSet(g_pInputDevice->evbit, EV_ABS);
+    ASMBitSet(g_pInputDevice->evbit, EV_KEY);
+# ifdef EV_SYN
+    ASMBitSet(g_pInputDevice->evbit, EV_SYN);
+# endif
+    input_set_abs_params(g_pInputDevice, ABS_X, RANGE_MIN, RANGE_MAX, 0, 0);
+    input_set_abs_params(g_pInputDevice, ABS_Y, RANGE_MIN, RANGE_MAX, 0, 0);
+    ASMBitSet(g_pInputDevice->keybit, BTN_MOUSE);
+    /** @todo this string should be in a header file somewhere. */
+    g_pInputDevice->name = "VirtualBox mouse integration";
+    return 0;
+}
+
+
+/**
+ * Terminates the kernel input device.
+ */
+static void vboxguestLinuxTermInputDevice(void)
+{
+    VbglGRFree(&g_pMouseStatusReq->header);
+    input_unregister_device(g_pInputDevice);
+    input_free_device(g_pInputDevice);
+}
+#endif
+
+
 /**
  * Creates the device nodes.
  *
@@ -436,7 +564,6 @@ static void vboxguestLinuxTermDeviceNodes(void)
 }
 
 
-
 /**
  * Initialize module.
  *
@@ -515,20 +642,46 @@ static int __init vboxguestLinuxModInit(void)
             if (RT_SUCCESS(rc))
             {
                 /*
-                 * Finally, create the device nodes.
+                 * Create the kernel session for this driver.
                  */
-                rc = vboxguestLinuxInitDeviceNodes();
-                if (rc >= 0)
+                rc = VBoxGuestCreateKernelSession(&g_DevExt,
+                                                  &g_pKernelSession);
+                if (RT_SUCCESS(rc))
                 {
-                    /* some useful information for the user but don't show this on the console */
-                    LogRel((DEVICE_NAME ": major %d, IRQ %d, I/O port %RTiop, MMIO at %RHp (size 0x%x)\n",
-                            g_iModuleMajor, g_pPciDev->irq, g_IOPortBase, g_MMIOPhysAddr, g_cbMMIO));
-                    printk(KERN_DEBUG DEVICE_NAME ": Successfully loaded version "
-                           VBOX_VERSION_STRING " (interface " RT_XSTR(VMMDEV_VERSION) ")\n");
-                    return rc;
-                }
+                    /*
+                     * Create the kernel input device.
+                     */
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+                    rc = vboxguestLinuxCreateInputDevice();
+                    if (rc >= 0)
+                    {
+#endif
+                        /*
+                         * Finally, create the device nodes.
+                         */
+                        rc = vboxguestLinuxInitDeviceNodes();
+                        if (rc >= 0)
+                        {
+                            /* some useful information for the user but don't show this on the console */
+                            LogRel((DEVICE_NAME ": major %d, IRQ %d, I/O port %RTiop, MMIO at %RHp (size 0x%x)\n",
+                                    g_iModuleMajor, g_pPciDev->irq, g_IOPortBase, g_MMIOPhysAddr, g_cbMMIO));
+                            printk(KERN_DEBUG DEVICE_NAME ": Successfully loaded version "
+                                   VBOX_VERSION_STRING " (interface " RT_XSTR(VMMDEV_VERSION) ")\n");
+                            return rc;
+                        }
 
-                /* bail out */
+                        /* bail out */
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+                        vboxguestLinuxTermInputDevice();
+                    }
+                    else
+                    {
+                        LogRel((DEVICE_NAME ": vboxguestCreateInputDevice failed with rc=%Rrc\n", rc));
+                        rc = RTErrConvertFromErrno(rc);
+                    }
+#endif
+                    VBoxGuestCloseSession(&g_DevExt, g_pKernelSession);
+                }
                 VBoxGuestDeleteDevExt(&g_DevExt);
             }
             else
@@ -556,6 +709,10 @@ static void __exit vboxguestLinuxModExit(void)
      * Inverse order of init.
      */
     vboxguestLinuxTermDeviceNodes();
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+    vboxguestLinuxTermInputDevice();
+#endif
+    VBoxGuestCloseSession(&g_DevExt, g_pKernelSession);
     VBoxGuestDeleteDevExt(&g_DevExt);
     vboxguestLinuxTermISR();
     vboxguestLinuxTermPci();
@@ -775,6 +932,9 @@ static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, lof
 
 void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 {
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+    int rc;
+#endif
     NOREF(pDevExt);
 
     /*
@@ -785,6 +945,23 @@ void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
     wake_up_all(&g_PollEventQueue);
     Log(("VBoxGuestNativeISRMousePollEvent: kill_fasync\n"));
     kill_fasync(&g_pFAsyncQueue, SIGIO, POLL_IN);
+#ifdef VBOXGUEST_WITH_INPUT_DRIVER
+    /* Report events to the kernel input device */
+    g_pMouseStatusReq->mouseFeatures = 0;
+    g_pMouseStatusReq->pointerXPos = 0;
+    g_pMouseStatusReq->pointerYPos = 0;
+    rc = VbglGRPerform(&g_pMouseStatusReq->header);
+    if (RT_SUCCESS(rc))
+    {
+        input_report_abs(g_pInputDevice, ABS_X,
+                         g_pMouseStatusReq->pointerXPos);
+        input_report_abs(g_pInputDevice, ABS_Y,
+                         g_pMouseStatusReq->pointerYPos);
+# ifdef EV_SYN
+        input_sync(g_pInputDevice);
+# endif
+    }
+#endif
     Log(("VBoxGuestNativeISRMousePollEvent: done\n"));
 }
 
