@@ -198,6 +198,7 @@ static int sf_read_super_aux(struct super_block *sb, void *data, int flags)
     struct sf_glob_info *sf_g;
     SHFLFSOBJINFO fsinfo;
     struct vbsf_mount_info_new *info;
+    bool fInodePut = true;
 
     TRACE();
     if (!data)
@@ -294,11 +295,18 @@ static int sf_read_super_aux(struct super_block *sb, void *data, int flags)
     unlock_new_inode(iroot);
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 4, 0)
+    droot = d_make_root(iroot);
+#else
     droot = d_alloc_root(iroot);
+#endif
     if (!droot)
     {
         err = -ENOMEM;  /* XXX */
         LogFunc(("d_alloc_root failed\n"));
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 4, 0)
+        fInodePut = false;
+#endif
         goto fail5;
     }
 
@@ -310,7 +318,8 @@ fail5:
     sf_done_backing_dev(sf_g);
 
 fail4:
-    iput(iroot);
+    if (fInodePut)
+        iput(iroot);
 
 fail3:
     kfree(sf_i->path);
@@ -487,7 +496,7 @@ static struct file_system_type vboxsf_fs_type =
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 static int follow_symlinks = 0;
-module_param(follow_symlinks, bool, 0);
+module_param(follow_symlinks, int, 0);
 MODULE_PARM_DESC(follow_symlinks, "Let host resolve symlinks rather than showing them");
 #endif
 
