@@ -1816,7 +1816,7 @@ const PDMDRVREG g_DrvIntNet =
     /* fClass. */
     PDM_DRVREG_CLASS_NETWORK,
     /* cMaxInstances */
-    ~0,
+    ~0U,
     /* cbInstance */
     sizeof(DRVINTNET),
     /* pfnConstruct */
