@@ -606,7 +606,13 @@ int slirp_init(PNATState *ppData, uint32_t u32NetAddr, uint32_t u32Netmask,
     pData->socket_snd = 64 * _1K;
     tcp_sndspace = 64 * _1K;
     tcp_rcvspace = 64 * _1K;
-    pData->soMaxConn = 1; /* historical value */
+    /**
+     * Assignment here has only meaning, to avoid additional noise in release log.
+     * The value's assigned from DrvNAT in function slirp_set_somaxconi by reading value of CFGM key
+     * "VBoxInternal/Devices/<adapter name>/0/LUN#0/Config/SoMaxConnection" or to
+     * default value 10 (xTracker/5983) in case value for the key wasn't found.
+     */
+    pData->soMaxConn = 10;
 
 #ifdef RT_OS_WINDOWS
     {
@@ -779,7 +785,18 @@ void slirp_term(PNATState pData)
     ftp_alias_unload(pData);
     nbt_alias_unload(pData);
     if (pData->fUseHostResolver)
+    {
         dns_alias_unload(pData);
+#ifdef VBOX_WITH_DNSMAPPING_IN_HOSTRESOLVER
+        while (!LIST_EMPTY(&pData->DNSMapHead))
+        {
+            PDNSMAPPINGENTRY pDnsEntry = LIST_FIRST(&pData->DNSMapHead);
+            LIST_REMOVE(pDnsEntry, MapList);
+            RTStrFree(pDnsEntry->pszCName);
+            RTMemFree(pDnsEntry);
+        }
+#endif
+    }
     while (!LIST_EMPTY(&instancehead))
     {
         struct libalias *la = LIST_FIRST(&instancehead);
@@ -1991,13 +2008,27 @@ void slirp_set_dhcp_dns_proxy(PNATState pData, bool fDNSProxy)
 void slirp_set_somaxconn(PNATState pData, int iSoMaxConn)
 {
     LogFlowFunc(("iSoMaxConn:d\n", iSoMaxConn));
+    /* Conditions */
     if (iSoMaxConn > SOMAXCONN)
     {
-        LogRel(("New value of somaxconn(%d) bigger than SOMAXCONN(%d)\n", iSoMaxConn, SOMAXCONN));
+        LogRel(("NAT: value of somaxconn(%d) bigger than SOMAXCONN(%d)\n", iSoMaxConn, SOMAXCONN));
         pData->soMaxConn = SOMAXCONN;
     }
-    pData->soMaxConn = iSoMaxConn > 0 ? iSoMaxConn : pData->soMaxConn;
-    LogRel(("New value of somaxconn: %d\n", pData->soMaxConn));
+
+    if (iSoMaxConn < 1)
+    {
+        LogRel(("NAT: proposed value(%d) of somaxconn is invalid, default value is used (%d)\n", iSoMaxConn, pData->soMaxConn));
+        LogFlowFuncLeave();
+        return;
+    }
+
+    /* Asignment */
+    if (pData->soMaxConn != iSoMaxConn)
+    {
+        LogRel(("NAT: value of somaxconn has been changed from %d to %d\n",
+                pData->soMaxConn, iSoMaxConn));
+        pData->soMaxConn = iSoMaxConn;
+    }
     LogFlowFuncLeave();
 }
 /* don't allow user set less 8kB and more than 1M values */
@@ -2109,6 +2140,43 @@ void slirp_arp_who_has(PNATState pData, uint32_t dst)
     if_encap(pData, ETH_P_ARP, m, ETH_ENCAP_URG);
     LogFlowFuncLeave();
 }
+#ifdef VBOX_WITH_DNSMAPPING_IN_HOSTRESOLVER
+void  slirp_add_host_resolver_mapping(PNATState pData, const char *pszHostName, const char *pszHostNamePattern, uint32_t u32HostIP)
+{
+    LogFlowFunc(("ENTER: pszHostName:%s, pszHostNamePattern:%s u32HostIP:%RTnaipv4\n",
+                pszHostName ? pszHostName : "(null)",
+                pszHostNamePattern ? pszHostNamePattern : "(null)",
+                u32HostIP));
+    if (   (   pszHostName
+            || pszHostNamePattern)
+        && u32HostIP != INADDR_ANY
+        && u32HostIP != INADDR_BROADCAST)
+    {
+        PDNSMAPPINGENTRY pDnsMapping = RTMemAllocZ(sizeof(DNSMAPPINGENTRY));
+        if (!pDnsMapping)
+        {
+            LogFunc(("Can't allocate DNSMAPPINGENTRY\n"));
+            LogFlowFuncLeave();
+            return;
+        }
+        pDnsMapping->u32IpAddress = u32HostIP;
+        if (pszHostName)
+            pDnsMapping->pszCName = RTStrDup(pszHostName);
+        else if (pszHostNamePattern)
+            pDnsMapping->pszPattern = RTStrDup(pszHostNamePattern);
+        if (   !pDnsMapping->pszCName
+            && !pDnsMapping->pszPattern)
+        {
+            LogFunc(("Can't allocate enough room for %s\n", pszHostName ? pszHostName : pszHostNamePattern));
+            RTMemFree(pDnsMapping);
+            LogFlowFuncLeave();
+            return;
+        }
+        LIST_INSERT_HEAD(&pData->DNSMapHead, pDnsMapping, MapList);
+    }
+    LogFlowFuncLeave();
+}
+#endif
 
 /* updates the arp cache
  * @note: this is helper function, slirp_arp_cache_update_or_add should be used.
