@@ -65,6 +65,22 @@ struct dns_domain_entry
 };
 LIST_HEAD(dns_domain_list_head, dns_domain_entry);
 
+#ifdef VBOX_WITH_DNSMAPPING_IN_HOSTRESOLVER
+typedef struct DNSMAPPINGENTRY
+{
+    /*
+     * host name to map
+     * Note: if pszCName isn't null pszPattern won't be used (see alias_dns.c for details)
+     */
+    char        *pszCName;
+    /* pattern of hostnames to map to specifaied IP */
+    char        *pszPattern;
+    uint32_t    u32IpAddress;
+    LIST_ENTRY(DNSMAPPINGENTRY) MapList;
+} DNSMAPPINGENTRY, *PDNSMAPPINGENTRY;
+typedef LIST_HEAD(DNSMAPPINGLISTHEAD, DNSMAPPINGENTRY) DNSMAPPINGLISTHEAD;
+#endif
+
 struct dns_entry
 {
     struct in_addr de_addr;
@@ -197,6 +213,8 @@ typedef struct NATState
 #  define NSOCK_INC_EX(ex) do {} while (0)
 #  define NSOCK_DEC_EX(ex) do {} while (0)
 # endif
+    int cIcmpCacheSize;
+    int iIcmpCacheLimit;
 # ifdef RT_OS_WINDOWS
     void *pvIcmpBuffer;
     size_t szIcmpBuffer;
@@ -290,6 +308,9 @@ typedef struct NATState
     struct proto_handler *nbt_module;
     struct proto_handler *dns_module;
 
+#ifdef VBOX_WITH_DNSMAPPING_IN_HOSTRESOLVER
+    DNSMAPPINGLISTHEAD DNSMapHead;
+#endif
 } NATState;
 
 
