@@ -1,4 +1,4 @@
-/* $Id: DrvSCSI.cpp 35353 2010-12-27 17:25:52Z vboxsync $ */
+/* $Id: DrvSCSI.cpp $ */
 /** @file
  * VBox storage drivers: Generic SCSI command parser and execution driver
  */
@@ -863,7 +863,7 @@ const PDMDRVREG g_DrvSCSI =
     /* fClass. */
     PDM_DRVREG_CLASS_SCSI,
     /* cMaxInstances */
-    ~0,
+    ~0U,
     /* cbInstance */
     sizeof(DRVSCSI),
     /* pfnConstruct */
