@@ -509,7 +509,7 @@ const PDMDRVREG g_DrvSCSIHost =
     /* fClass. */
     PDM_DRVREG_CLASS_SCSI,
     /* cMaxInstances */
-    ~0,
+    ~0U,
     /* cbInstance */
     sizeof(DRVSCSIHOST),
     /* pfnConstruct */
