@@ -188,13 +188,15 @@ int handleControlVM(HandlerArg *a)
                     /* check if we are already paused */
                     MachineState_T machineState;
                     CHECK_ERROR_BREAK(console, COMGETTER(State)(&machineState));
+                    /* the error code was lost by the previous instruction */
+                    rc = VBOX_E_INVALID_VM_STATE;
                     if (machineState != MachineState_Paused)
                     {
                         RTMsgError("Machine in invalid state %d -- %s\n",
                                    machineState, machineStateToName(machineState, false));
-                        break;
                     }
                 }
+                break;
             }
 
             ComPtr<IProgress> progress;
