@@ -1,10 +1,10 @@
-/* $Id: VBoxManageDisk.cpp 37525 2011-06-17 10:09:21Z vboxsync $ */
+/* $Id: VBoxManageDisk.cpp $ */
 /** @file
  * VBoxManage - The disk related commands.
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -658,6 +658,7 @@ int handleCloneHardDisk(HandlerArg *a)
             rc = createHardDisk(a, Utf8Str(format).c_str(), pszDst, dstDisk);
             if (FAILED(rc))
                 break;
+            fDstUnknown = true;
         }
 
         ComPtr<IProgress> progress;
@@ -704,6 +705,7 @@ static const RTGETOPTDEF g_aConvertFromRawHardDiskOptions[] =
     { "-static",        'F', RTGETOPT_REQ_NOTHING },
     { "--variant",      'm', RTGETOPT_REQ_STRING },
     { "-variant",       'm', RTGETOPT_REQ_STRING },
+    { "--uuid",         'u', RTGETOPT_REQ_STRING },
 };
 
 RTEXITCODE handleConvertFromRaw(int argc, char *argv[])
@@ -716,6 +718,8 @@ RTEXITCODE handleConvertFromRaw(int argc, char *argv[])
     const char *filesize = NULL;
     unsigned uImageFlags = VD_IMAGE_FLAGS_NONE;
     void *pvBuf = NULL;
+    RTUUID uuid;
+    PCRTUUID pUuid = NULL;
 
     int c;
     RTGETOPTUNION ValueUnion;
@@ -727,6 +731,11 @@ RTEXITCODE handleConvertFromRaw(int argc, char *argv[])
     {
         switch (c)
         {
+            case 'u':   // --uuid
+                if (RT_FAILURE(RTUuidFromStr(&uuid, ValueUnion.psz)))
+                    return errorSyntax(USAGE_CONVERTFROMRAW, "Invalid UUID '%s'", ValueUnion.psz);
+                pUuid = &uuid;
+                break;
             case 'o':   // --format
                 format = ValueUnion.psz;
                 break;
@@ -826,7 +835,7 @@ RTEXITCODE handleConvertFromRaw(int argc, char *argv[])
     LCHS.cHeads = 0;
     LCHS.cSectors = 0;
     rc = VDCreateBase(pDisk, format, dstfilename, cbFile,
-                      uImageFlags, pszComment, &PCHS, &LCHS, NULL,
+                      uImageFlags, pszComment, &PCHS, &LCHS, pUuid,
                       VD_OPEN_FLAGS_NORMAL, NULL, NULL);
     if (RT_FAILURE(rc))
     {
