@@ -1,4 +1,4 @@
-/* $Id: main.cpp 38324 2011-08-05 14:02:53Z vboxsync $ */
+/* $Id: main.cpp $ */
 /** @file
  *
  * VBox frontends: Qt GUI ("VirtualBox"):
@@ -76,8 +76,8 @@ QString g_QStrHintLinuxNoDriver = QApplication::tr(
   "there is a permission problem with /dev/vboxdrv. Please reinstall the kernel "
   "module by executing<br/><br/>"
   "  <font color=blue>'/etc/init.d/vboxdrv setup'</font><br/><br/>"
-  "as root. Users of Ubuntu, Fedora or Mandriva should install the DKMS "
-  "package first. This package keeps track of Linux kernel changes and "
+  "as root. If it is available in your distribution, you should install the "
+  "DKMS package first. This package keeps track of Linux kernel changes and "
   "recompiles the vboxdrv kernel module if necessary."
   );
 
@@ -525,9 +525,6 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 #ifdef VBOX_WITH_REGISTRATION_REQUEST
                     vboxGlobal().showRegistrationDialog (false /* aForce */);
 #endif
-#ifdef VBOX_WITH_UPDATE_REQUEST
-                    vboxGlobal().showUpdateDialog (false /* aForce */);
-#endif
 #ifdef VBOX_GUI_WITH_SYSTRAY
                 }
 
