@@ -104,9 +104,11 @@ static SUPINSTFILE const    g_aSupInstallFiles[] =
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDDR0.r0" },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDD2R0.r0" },
 
+#ifdef VBOX_WITH_RAW_MODE
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VMMGC.gc" },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDDGC.gc" },
     {   kSupIFT_Dll,  kSupID_AppPrivArch,       false, "VBoxDD2GC.gc" },
+#endif
 
     {   kSupIFT_Dll,  kSupID_SharedLib,         false, "VBoxRT" SUPLIB_DLL_SUFF },
     {   kSupIFT_Dll,  kSupID_SharedLib,         false, "VBoxVMM" SUPLIB_DLL_SUFF },
@@ -277,7 +279,7 @@ static int supR3HardenedMakeFilePath(PCSUPINSTFILE pFile, char *pszDst, size_t c
      * Combine supR3HardenedMakePath and the filename.
      */
     int rc = supR3HardenedMakePath(pFile->enmDir, pszDst, cchDst, fFatal);
-    if (RT_SUCCESS(rc))
+    if (RT_SUCCESS(rc) && fWithFilename)
     {
         size_t cchFile = strlen(pFile->pszFile);
         size_t off = strlen(pszDst);
@@ -455,7 +457,7 @@ static int supR3HardenedVerifyFileInternal(int iFile, bool fFatal, bool fLeaveFi
     if (RT_SUCCESS(rc))
     {
         char szPath[RTPATH_MAX];
-        rc = supR3HardenedMakeFilePath(pFile, szPath, sizeof(szPath), true, fFatal);
+        rc = supR3HardenedMakeFilePath(pFile, szPath, sizeof(szPath), true /*fWithFilename*/, fFatal);
         if (RT_SUCCESS(rc))
         {
 #if defined(RT_OS_WINDOWS)
@@ -813,6 +815,7 @@ static int supR3HardenedSetError3(int rc, PRTERRINFO pErrInfo, const char *pszMs
     return supR3HardenedSetErrorN(rc, pErrInfo, 3, pszMsg1, pszMsg2, pszMsg3);
 }
 
+#ifdef SOME_UNUSED_FUNCTION
 
 /**
  * Copies the two messages into the error buffer and returns @a rc.
@@ -843,6 +846,7 @@ static int supR3HardenedSetError(int rc, PRTERRINFO pErrInfo, const char *pszMsg
     return supR3HardenedSetErrorN(rc, pErrInfo, 1, pszMsg);
 }
 
+#endif /* SOME_UNUSED_FUNCTION */
 
 /**
  * Output from a successfull supR3HardenedVerifyPathSanity call.
@@ -1125,8 +1129,8 @@ static int supR3HardenedIsSameFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState1, P
  * @param   fRelaxed            Whether we can be more relaxed about this
  *                              directory (only used for grand parent
  *                              directories).
- * @param   pszPath             The path to the object. (For error messages
- *                              only.)
+ * @param   pszPath             The path to the object. For error messages and
+ *                              securing a couple of hacks.
  * @param   pErrInfo            The error info structure.
  */
 static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bool fDir, bool fRelaxed,
@@ -1134,10 +1138,12 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
 {
 #if defined(RT_OS_WINDOWS)
     /** @todo Windows hardening. */
+    NOREF(pFsObjState); NOREF(fDir); NOREF(fRelaxed); NOREF(pszPath); NOREF(pErrInfo);
     return VINF_SUCCESS;
 
 #elif defined(RT_OS_OS2)
     /* No hardening here - it's a single user system. */
+    NOREF(pFsObjState); NOREF(fDir); NOREF(fRelaxed); NOREF(pszPath); NOREF(pErrInfo);
     return VINF_SUCCESS;
 
 #else
@@ -1185,12 +1191,22 @@ static int supR3HardenedVerifyFsObject(PCSUPR3HARDENEDFSOBJSTATE pFsObjState, bo
         /* HACK ALERT: On Darwin /Applications is root:admin with admin having
            full access. So, to work around we relax the hardening a bit and
            permit grand parents and beyond to be group writable by admin. */
-        if (pFsObjState->Stat.st_gid != 80 /*admin*/) /** @todo dynamically resolve the admin group? */
+        /** @todo dynamically resolve the admin group? */
+        bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 80 /*admin*/ || strcmp(pszPath, "/Applications");
+
 #elif defined(RT_OS_FREEBSD)
-        /* HACK ALERT: PC-BSD 9 has group-writable application directory,
-           similar to OS X and their /Applications directory (see above). */
-        if (pFsObjState->Stat.st_gid != 5 /*operators*/)
+        /* HACK ALERT: PC-BSD 9 has group-writable /usr/pib directory which is
+           similar to /Applications on OS X (see above).
+           On FreeBSD root is normally the only member of this group, on
+           PC-BSD the default user is a member. */
+        /** @todo dynamically resolve the operator group? */
+        bool fBad = !fRelaxed || pFsObjState->Stat.st_gid != 5 /*operator*/ || strcmp(pszPath, "/usr/pbi");
+        NOREF(fRelaxed);
+#else
+        NOREF(fRelaxed);
+        bool fBad = true;
 #endif
+        if (fBad)
             return supR3HardenedSetError3(VERR_SUPLIB_WRITE_NON_SYS_GROUP, pErrInfo,
                                           "The group is not a system group and it has write access to '", pszPath, "'");
     }
