@@ -116,7 +116,11 @@ GLint crServerDispatchCreateContextEx(const char *dpyName, GLint visualBits, GLi
         err = cr_server.head_spu->dispatch_table.GetError();
         if (err!=GL_NO_ERROR)
         {
+#ifdef DEBUG_misha
+            crDebug("Cleared gl error %#x on context creation", err);
+#else
             crWarning("Cleared gl error %#x on context creation", err);
+#endif
         }
     }
 
@@ -226,7 +230,7 @@ void SERVER_DISPATCH_APIENTRY
 crServerDispatchMakeCurrent( GLint window, GLint nativeWindow, GLint context )
 {
     CRMuralInfo *mural, *oldMural;
-    CRContext *ctx;
+    CRContext *ctx, *oldCtx;
 
     if (context >= 0 && window >= 0) {
         mural = (CRMuralInfo *) crHashtableSearch(cr_server.muralTable, window);
@@ -263,6 +267,13 @@ crServerDispatchMakeCurrent( GLint window, GLint nativeWindow, GLint context )
         return;
     }
 
+    /* Ubuntu 11.04 hosts misbehave if context window switch is
+     * done with non-default framebuffer object settings.
+     * crStateSwichPrepare & crStateSwichPostprocess are supposed to work around this problem
+     * crStateSwichPrepare restores the FBO state to its default values before the context window switch,
+     * while crStateSwichPostprocess restores it back to the original values */
+    oldCtx = crStateSwichPrepare(ctx);
+
     /*
     crDebug("**** %s client %d  curCtx=%d curWin=%d", __func__,
                     cr_server.curClient->number, ctxPos, window);
@@ -321,6 +332,8 @@ crServerDispatchMakeCurrent( GLint window, GLint nativeWindow, GLint context )
     /* This used to be earlier, after crStateUpdateColorBits() call */
     crStateMakeCurrent( ctx );
 
+    crStateSwichPostprocess(oldCtx);
+
     if (oldMural != mural && crServerSupportRedirMuralFBO())
     {
         if (!crStateGetCurrent()->framebufferobject.drawFB)
