@@ -221,6 +221,7 @@ public:
     EventSource *getEventSource() { return mEventSource; }
 
     int VRDPClientLogon(uint32_t u32ClientId, const char *pszUser, const char *pszPassword, const char *pszDomain);
+    void VRDPClientStatusChange(uint32_t u32ClientId, const char *pszStatus);
     void VRDPClientConnect(uint32_t u32ClientId);
     void VRDPClientDisconnect(uint32_t u32ClientId, uint32_t fu32Intercepted);
     void VRDPInterceptAudio(uint32_t u32ClientId);
@@ -228,6 +229,20 @@ public:
     void VRDPInterceptClipboard(uint32_t u32ClientId);
 
     void processRemoteUSBDevices(uint32_t u32ClientId, VRDEUSBDEVICEDESC *pDevList, uint32_t cbDevList, bool fDescExt);
+    void reportGuestStatistics(ULONG aValidStats, ULONG aCpuUser,
+                               ULONG aCpuKernel, ULONG aCpuIdle,
+                               ULONG aMemTotal, ULONG aMemFree,
+                               ULONG aMemBalloon, ULONG aMemShared,
+                               ULONG aMemCache, ULONG aPageTotal,
+                               ULONG aAllocVMM, ULONG aFreeVMM,
+                               ULONG aBalloonedVMM, ULONG aSharedVMM)
+    {
+        mControl->ReportGuestStatistics(aValidStats, aCpuUser, aCpuKernel, aCpuIdle,
+                                        aMemTotal, aMemFree, aMemBalloon, aMemShared,
+                                        aMemCache, aPageTotal, aAllocVMM, aFreeVMM,
+                                        aBalloonedVMM, aSharedVMM);
+    }
+    void enableVMMStatistics(BOOL aEnable);
 
     // callback callers (partly; for some events console callbacks are notified
     // directly from IInternalSessionControl event handlers declared above)
@@ -660,6 +675,9 @@ private:
     void guestPropertiesHandleVMReset(void);
     bool guestPropertiesVRDPEnabled(void);
     void guestPropertiesVRDPUpdateLogon(uint32_t u32ClientId, const char *pszUser, const char *pszDomain);
+    void guestPropertiesVRDPUpdateActiveClient(uint32_t u32ClientId);
+    void guestPropertiesVRDPUpdateClientAttach(uint32_t u32ClientId, bool fAttached);
+    void guestPropertiesVRDPUpdateNameChange(uint32_t u32ClientId, const char *pszName);
     void guestPropertiesVRDPUpdateDisconnect(uint32_t u32ClientId);
 #endif
 
