@@ -582,7 +582,7 @@ public:
      * used by ready Machine children (whose readiness is bound to the parent's
      * one) or after doing addCaller() manually.
      */
-    bool isAccessible() const { return mData->mAccessible; }
+    bool isAccessible() const { return !!mData->mAccessible; }
 
     /**
      * Returns this machine ID.
@@ -1015,6 +1015,13 @@ public:
     STDMETHOD(UnlockMedia)() { unlockMedia(); return S_OK; }
     STDMETHOD(EjectMedium)(IMediumAttachment *aAttachment,
                            IMediumAttachment **aNewAttachment);
+    STDMETHOD(ReportGuestStatistics)(ULONG aValidStats, ULONG aCpuUser,
+                                     ULONG aCpuKernel, ULONG aCpuIdle,
+                                     ULONG aMemTotal, ULONG aMemFree,
+                                     ULONG aMemBalloon, ULONG aMemShared,
+                                     ULONG aMemCache, ULONG aPageTotal,
+                                     ULONG aAllocVMM, ULONG aFreeVMM,
+                                     ULONG aBalloonedVMM, ULONG aSharedVMM);
 
     // public methods only for internal purposes
 
