@@ -1,4 +1,4 @@
-/* $Id: zip.cpp 33982 2010-11-11 12:37:18Z vboxsync $ */
+/* $Id: zip.cpp $ */
 /** @file
  * IPRT - Compression.
  */
@@ -419,25 +419,37 @@ static DECLCALLBACK(int) rtZipStoreDecompInit(PRTZIPDECOMP pZip)
 /**
  * Convert from zlib errno to iprt status code.
  * @returns iprt status code.
- * @param   rc      Zlib error code.
+ * @param   rc              Zlib error code.
+ * @param   fCompressing    Set if we're compressing, clear if decompressing.
  */
-static int zipErrConvertFromZlib(int rc)
+static int zipErrConvertFromZlib(int rc, bool fCompressing)
 {
-    /** @todo proper zlib error conversion. */
     switch (rc)
     {
-        case Z_ERRNO:
-            return RTErrConvertFromErrno(errno);
+        case Z_OK:
+            return VINF_SUCCESS;
+
         case Z_STREAM_ERROR:
+            return VERR_ZIP_CORRUPTED;
+
         case Z_DATA_ERROR:
+            return fCompressing ? VERR_ZIP_ERROR : VERR_ZIP_CORRUPTED;
+
         case Z_MEM_ERROR:
+            return VERR_ZIP_NO_MEMORY;
+
         case Z_BUF_ERROR:
+            return VERR_ZIP_ERROR;
+
         case Z_VERSION_ERROR:
-            return VERR_GENERAL_FAILURE;
+            return VERR_ZIP_UNSUPPORTED_VERSION;
+
+        case Z_ERRNO: /* We shouldn't see this status! */
         default:
+            AssertMsgFailed(("%d\n", rc));
             if (rc >= 0)
                 return VINF_SUCCESS;
-            return VERR_GENERAL_FAILURE;
+            return VERR_ZIP_ERROR;
     }
 }
 
@@ -468,7 +480,7 @@ static DECLCALLBACK(int) rtZipZlibCompress(PRTZIPCOMP pZip, const void *pvBuf, s
          */
         int rc = deflate(&pZip->u.Zlib, Z_NO_FLUSH);
         if (rc != Z_OK)
-            return zipErrConvertFromZlib(rc);
+            return zipErrConvertFromZlib(rc, true /*fCompressing*/);
     }
     return VINF_SUCCESS;
 }
@@ -501,7 +513,7 @@ static DECLCALLBACK(int) rtZipZlibCompFinish(PRTZIPCOMP pZip)
          */
         rc = deflate(&pZip->u.Zlib, Z_FINISH);
         if (rc != Z_OK && rc != Z_STREAM_END)
-            return zipErrConvertFromZlib(rc);
+            return zipErrConvertFromZlib(rc, true /*fCompressing*/);
     }
     return VINF_SUCCESS;
 }
@@ -517,7 +529,7 @@ static DECLCALLBACK(int) rtZipZlibCompDestroy(PRTZIPCOMP pZip)
      */
     int rc = deflateEnd(&pZip->u.Zlib);
     if (rc != Z_OK)
-        rc = zipErrConvertFromZlib(rc);
+        rc = zipErrConvertFromZlib(rc, true /*fCompressing*/);
     return rc;
 }
 
@@ -549,7 +561,7 @@ static DECLCALLBACK(int) rtZipZlibCompInit(PRTZIPCOMP pZip, RTZIPLEVEL enmLevel)
     pZip->u.Zlib.opaque    = pZip;
 
     int rc = deflateInit(&pZip->u.Zlib, enmLevel);
-    return rc >= 0 ? rc = VINF_SUCCESS : zipErrConvertFromZlib(rc);
+    return rc >= 0 ? rc = VINF_SUCCESS : zipErrConvertFromZlib(rc, true /*fCompressing*/);
 }
 
 
@@ -594,7 +606,7 @@ static DECLCALLBACK(int) rtZipZlibDecompress(PRTZIPDECOMP pZip, void *pvBuf, siz
             break;
         }
         if (rc != Z_OK)
-            return zipErrConvertFromZlib(rc);
+            return zipErrConvertFromZlib(rc, false /*fCompressing*/);
     }
     return VINF_SUCCESS;
 }
@@ -610,7 +622,7 @@ static DECLCALLBACK(int) rtZipZlibDecompDestroy(PRTZIPDECOMP pZip)
      */
     int rc = inflateEnd(&pZip->u.Zlib);
     if (rc != Z_OK)
-        rc = zipErrConvertFromZlib(rc);
+        rc = zipErrConvertFromZlib(rc, false /*fCompressing*/);
     return rc;
 }
 
@@ -629,7 +641,7 @@ static DECLCALLBACK(int) rtZipZlibDecompInit(PRTZIPDECOMP pZip)
     pZip->u.Zlib.opaque    = pZip;
 
     int rc = inflateInit(&pZip->u.Zlib);
-    return rc >= 0 ? VINF_SUCCESS : zipErrConvertFromZlib(rc);
+    return rc >= 0 ? VINF_SUCCESS : zipErrConvertFromZlib(rc, false /*fCompressing*/);
 }
 
 #endif /* RTZIP_USE_ZLIB */
