@@ -57,14 +57,14 @@
  */
 static const unsigned g_aSize2Shift[] =
 {
-    ~0,    /* 0 - invalid */
+    ~0U,   /* 0 - invalid */
     0,     /* *1 == 2^0 */
     1,     /* *2 == 2^1 */
-    ~0,    /* 3 - invalid */
+    ~0U,   /* 3 - invalid */
     2,     /* *4 == 2^2 */
-    ~0,    /* 5 - invalid */
-    ~0,    /* 6 - invalid */
-    ~0,    /* 7 - invalid */
+    ~0U,   /* 5 - invalid */
+    ~0U,   /* 6 - invalid */
+    ~0U,   /* 7 - invalid */
     3      /* *8 == 2^3 */
 };
 
@@ -2329,7 +2329,8 @@ VMMDECL(VBOXSTRICTRC) IOMInterpretOUTS(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUS
  *
  * (This is a special optimization used by the VGA device.)
  *
- * @returns VBox status code.
+ * @returns VBox status code.  This API may return VINF_SUCCESS even if no
+ *          remapping is made,.
  *
  * @param   pVM             The virtual machine.
  * @param   GCPhys          The address of the MMIO page to be changed.
@@ -2350,7 +2351,9 @@ VMMDECL(int) IOMMMIOMapMMIO2Page(PVM pVM, RTGCPHYS GCPhys, RTGCPHYS GCPhysRemapp
              && !HWACCMIsNestedPagingActive(pVM)))
         return VINF_SUCCESS;    /* ignore */
 
-    IOM_LOCK(pVM);
+    int rc = IOM_LOCK(pVM);
+    if (RT_FAILURE(rc))
+        return VINF_SUCCESS; /* better luck the next time around */
 
     /*
      * Lookup the context range node the page belongs to.
@@ -2368,7 +2371,7 @@ VMMDECL(int) IOMMMIOMapMMIO2Page(PVM pVM, RTGCPHYS GCPhys, RTGCPHYS GCPhysRemapp
     GCPhys         &= ~(RTGCPHYS)PAGE_OFFSET_MASK;
     GCPhysRemapped &= ~(RTGCPHYS)PAGE_OFFSET_MASK;
 
-    int rc = PGMHandlerPhysicalPageAlias(pVM, pRange->GCPhys, GCPhys, GCPhysRemapped);
+    rc = PGMHandlerPhysicalPageAlias(pVM, pRange->GCPhys, GCPhys, GCPhysRemapped);
 
     IOM_UNLOCK(pVM);
     AssertRCReturn(rc, rc);
