@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2011 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -67,7 +67,7 @@ VMMR0DECL(void) ModuleTerm(void);
 #if defined(RT_ARCH_X86) && (defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD))
 extern uint64_t __udivdi3(uint64_t, uint64_t);
 extern uint64_t __umoddi3(uint64_t, uint64_t);
-#endif // RT_ARCH_X86 && (RT_OS_SOLARIS || RT_OS_FREEBSD)
+#endif
 RT_C_DECLS_END
 
 
@@ -83,7 +83,7 @@ PFNRT g_VMMGCDeps[] =
 #if defined(RT_ARCH_X86) && (defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD))
     (PFNRT)__udivdi3,
     (PFNRT)__umoddi3,
-#endif // RT_ARCH_X86 && (RT_OS_SOLARIS || RT_OS_FREEBSD)
+#endif
     NULL
 };
 
@@ -94,13 +94,6 @@ extern "C" { char _depends_on[] = "vboxdrv"; }
 
 
 
-#if defined(RT_OS_WINDOWS) && defined(RT_ARCH_AMD64)
-/* Increase the size of the image to work around the refusal of Win64 to
- * load images in the 0x80000 range.
- */
-static uint64_t u64BloatImage[8192] = {0};
-#endif
-
 /**
  * Initialize the module.
  * This is called when we're first loaded.
@@ -1085,13 +1078,21 @@ static int vmmR0EntryExWorker(PVM pVM, VMCPUID idCpu, VMMR0OPERATION enmOperatio
 
 #if defined(VBOX_STRICT) && HC_ARCH_BITS == 64
         case VMMR0_DO_GMM_FIND_DUPLICATE_PAGE:
-        {
             if (u64Arg)
                 return VERR_INVALID_PARAMETER;
             return GMMR0FindDuplicatePageReq(pVM, (PGMMFINDDUPLICATEPAGEREQ)pReqHdr);
-        }
 #endif
 
+        case VMMR0_DO_GMM_QUERY_STATISTICS:
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0QueryStatisticsReq(pVM, (PGMMQUERYSTATISTICSSREQ)pReqHdr);
+
+        case VMMR0_DO_GMM_RESET_STATISTICS:
+            if (u64Arg)
+                return VERR_INVALID_PARAMETER;
+            return GMMR0ResetStatisticsReq(pVM, (PGMMRESETSTATISTICSSREQ)pReqHdr);
+
         /*
          * A quick GCFGM mock-up.
          */
@@ -1250,7 +1251,7 @@ typedef VMMR0ENTRYEXARGS *PVMMR0ENTRYEXARGS;
  * @returns VBox status code.
  * @param   pvArgs      The argument package
  */
-static int vmmR0EntryExWrapper(void *pvArgs)
+static DECLCALLBACK(int) vmmR0EntryExWrapper(void *pvArgs)
 {
     return vmmR0EntryExWorker(((PVMMR0ENTRYEXARGS)pvArgs)->pVM,
                               ((PVMMR0ENTRYEXARGS)pvArgs)->idCpu,
